# Sample code: Get frame from webcam, run object detection on the frame, then display the frame
# Note: you will need to have the opencv-python module installed

import tfsdk
import cv2
import os
from threading import Thread, Lock
import time

# Utility class for grabbing RTSP stream frames
# Runs in alternate thread to ensure we always have the latest frame from our RTSP stream
class VideoController:
    def __init__(self):
        self.mutex = Lock()

        # Use the default camera (TODO: Can change the camera source, for example to an RTSP stream)
        self.cap = cv2.VideoCapture(0)
        if (self.cap.isOpened()== False): 
            print("Error opening video stream")
            os._exit(1)

        # Launch a thread to grab the newest frame from the frame buffer
        self.thread = Thread(target = self.rtspThreadFunc)
        self.thread.daemon = True
        self.thread.start()

    def rtspThreadFunc(self):
        fps = self.cap.get(cv2.CAP_PROP_FPS)
        sleepTimeSeconds = 1 / fps

        while True:
            # Sleep otherwise we won't give the main thread a chance to acquire the mutex
            time.sleep(0.001)
            
            # Grab a frame from the frame buffer, discard the return value
            # This will discard built up frames in the buffer to ensure we only process the latest frame to remove any delay
            self.mutex.acquire()
            self.cap.grab()
            self.mutex.release()

    # Returns true if a frame was grabbed
    def grabFrame(self):
        self.mutex.acquire()
        ret, image = self.cap.retrieve()
        self.mutex.release()
        return ret, image 


def draw_label(image, point, label,
               font=cv2.FONT_HERSHEY_SIMPLEX,
               font_scale=1.0, thickness=1):
    size = cv2.getTextSize(label, font, font_scale, thickness)[0]
    x_label, y_label = point
    cv2.rectangle(
        image,
        (x_label, y_label - size[1] - 10),
        (x_label + size[0], y_label),
        (194,134,58),
        cv2.FILLED)

    cv2.putText(
        image, label.capitalize(), (x_label, y_label - 5), font, font_scale,
        (255, 255, 255), thickness, cv2.LINE_AA)


# Create instance of the VideoController class
videoController = VideoController()

options = tfsdk.ConfigurationOptions()
# Can set configuration options here
# ex:
# options.smallest_face_height = 40
# options.fd_mode = tfsdk.FACEDETECTIONMODE.VERSATILE
# options.fd_filter = tfsdk.FACEDETECTIONFILTER.BALANCED
# options.fr_model = tfsdk.FACIALRECOGNITIONMODEL.FULL

# Use the accurate object detector
options.obj_model = tfsdk.OBJECTDETECTIONMODEL.ACCURATE 

sdk = tfsdk.SDK(options)

# TODO: export your license token as TRUEFACE_TOKEN environment variable
is_valid = sdk.set_license(os.environ['TRUEFACE_TOKEN'])
if (is_valid == False):
    print("Invalid License Provided")
    print("Be sure to export your license token as TRUEFACE_TOKEN")
    quit()

while(True):
    ret, frame = videoController.grabFrame()
    if ret == False:
        continue

    # Set the image using the frame buffer. OpenCV stores images in BGR format
    res = sdk.set_image(frame, frame.shape[1], frame.shape[0], tfsdk.COLORCODE.bgr)
    if (res != tfsdk.ERRORCODE.NO_ERROR):
        print("Unable to set frame")
        continue

    # Run object detection
    bounding_boxes = sdk.detect_objects()
    if (len(bounding_boxes) > 0) :
        
        # Draw the bounding boxes and label on the frame
        for bounding_box in bounding_boxes:
            tl = (int(bounding_box.top_left.x), int(bounding_box.top_left.y))
            br = (int(bounding_box.top_left.x + bounding_box.width), int(bounding_box.top_left.y + bounding_box.height))

            # Draw the rectangle on the frame
            cv2.rectangle(frame, tl, br, (194,134,58), 3)

            label_string = sdk.get_object_label_string(bounding_box.label)
            draw_label(frame, tl, label_string)


    # Display the resulting frame
    cv2.imshow('frame', frame)
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

# When everything done, release the capture
cap.release()
cv2.destroyAllWindows()




